# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.1

import enum

import winrt.system
from winui3._winui3_microsoft_graphics_imaging import (
    ImageBuffer,
)

__all__ = [
    "ImageBufferPixelFormat",
    "ImageBuffer",
]

class ImageBufferPixelFormat(enum.IntEnum):
    RGB8 = 137224
    RGBA8 = 30
    ARGB8 = 2498570
    BGRA8 = 87
    GRAY8 = 62

